import { Event, StateValue, Transition, Action, StandardMachine, ParallelMachine, SimpleOrCompoundStateNodeConfig, MachineConfig, ParallelMachineConfig, EventType, StandardMachineConfig, StateNodeConfig, Activity } from './types';
import matchesState from './matchesState';
import mapState from './mapState';
import { State } from './State';
declare class StateNode implements StateNodeConfig {
    config: SimpleOrCompoundStateNodeConfig | StandardMachineConfig | ParallelMachineConfig;
    key: string;
    id: string;
    relativeId: string;
    initial?: string;
    parallel?: boolean;
    states: Record<string, StateNode>;
    on?: Record<string, Transition | undefined>;
    onEntry?: Action[];
    onExit?: Action[];
    activities?: Activity[];
    strict: boolean;
    parent?: StateNode;
    machine: StateNode;
    data: object | undefined;
    private __cache;
    constructor(config: SimpleOrCompoundStateNodeConfig | StandardMachineConfig | ParallelMachineConfig);
    getStateNodes(state: StateValue | State): StateNode[];
    handles(event: Event): boolean;
    transition(state: StateValue | State, event: Event, extendedState?: any): State;
    private stateTransitionToState(stateTransition, prevState);
    private transitionStateValue(state, event, extendedState?);
    private next(event, history?, extendedState?);
    private readonly resolvedStateValue;
    private readonly initialStateValue;
    readonly initialState: State;
    getStates(stateValue: StateValue): StateNode[];
    getState(relativeStateId: string | string[]): StateNode | undefined;
    readonly events: EventType[];
    private getRelativeValue(toNode?);
}
export interface MachineFactory {
    (config: MachineConfig | ParallelMachineConfig): StandardMachine | ParallelMachine;
    standard: (config: MachineConfig) => StandardMachine;
    parallel: (config: ParallelMachineConfig) => ParallelMachine;
}
export declare function Machine(config: MachineConfig | ParallelMachineConfig): StandardMachine | ParallelMachine;
export { StateNode, State, matchesState, mapState };
