"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var xml_js_1 = require("xml-js");
var xstate = require("./index");
var utils_1 = require("./utils");
var pedestrianStates = {
    initial: 'walk',
    states: {
        walk: {
            on: {
                PED_COUNTDOWN: 'wait'
            }
        },
        wait: {
            on: {
                PED_COUNTDOWN: 'stop'
            }
        },
        stop: {}
    }
};
var lightMachine = xstate.Machine({
    key: 'light',
    initial: 'green',
    states: {
        green: {
            on: {
                TIMER: 'yellow',
                POWER_OUTAGE: 'red'
            }
        },
        yellow: {
            on: {
                TIMER: 'red',
                POWER_OUTAGE: 'red'
            }
        },
        red: __assign({ on: {
                TIMER: 'green',
                POWER_OUTAGE: 'red'
            } }, pedestrianStates)
    }
});
function stateNodeToSCXML(stateNode) {
    var parallel = stateNode.parallel;
    var scxmlElement = {
        type: 'element',
        name: parallel ? 'parallel' : 'state',
        attributes: {
            id: stateNode.id
        },
        elements: [
            !parallel && stateNode.initial
                ? {
                    type: 'element',
                    name: 'initial',
                    elements: [
                        {
                            type: 'element',
                            name: 'transition',
                            attributes: {
                                target: stateNode.states[stateNode.initial].id
                            }
                        }
                    ]
                }
                : undefined,
            stateNode.onEntry && {
                type: 'element',
                name: 'onentry',
                elements: stateNode.onEntry.map(function (event) {
                    return {
                        type: 'element',
                        name: 'send',
                        attributes: {
                            event: utils_1.getEventType(event)
                        }
                    };
                })
            },
            stateNode.onExit && {
                type: 'element',
                name: 'onexit',
                elements: stateNode.onExit.map(function (event) {
                    return {
                        type: 'element',
                        name: 'send',
                        attributes: {
                            event: utils_1.getEventType(event)
                        }
                    };
                })
            }
        ].concat(Object.keys(stateNode.states).map(function (stateKey) {
            var subStateNode = stateNode.states[stateKey];
            return stateNodeToSCXML(subStateNode);
        }), (stateNode.on
            ? Object.keys(stateNode.on)
                .map(function (event) {
                var transition = stateNode.on[event];
                if (!transition) {
                    return [];
                }
                if (Array.isArray(transition)) {
                    return transition.map(function (targetTransition) {
                        return {
                            type: 'element',
                            name: 'transition',
                            attributes: __assign({}, event ? { event: event } : undefined, { target: stateNode.parent.getState(targetTransition.target).id }, targetTransition.cond
                                ? { cond: targetTransition.cond.toString() }
                                : undefined),
                            elements: targetTransition.actions
                                ? targetTransition.actions.map(function (action) { return ({
                                    type: 'element',
                                    name: 'send',
                                    attributes: {
                                        event: utils_1.getEventType(action)
                                    }
                                }); })
                                : undefined
                        };
                    });
                }
                if (typeof transition === 'string') {
                    return [
                        {
                            type: 'element',
                            name: 'transition',
                            attributes: __assign({}, event ? { event: event } : undefined, { target: stateNode.parent.getState(stateNode.on[event]).id })
                        }
                    ];
                }
                return Object.keys(transition).map(function (target) {
                    var targetTransition = transition[target];
                    return {
                        type: 'element',
                        name: 'transition',
                        attributes: __assign({}, event ? { event: event } : undefined, { target: stateNode.parent.getState(target).id }, targetTransition.cond
                            ? { cond: targetTransition.cond.toString() }
                            : undefined),
                        elements: targetTransition.actions
                            ? targetTransition.actions.map(function (action) { return ({
                                type: 'element',
                                name: 'send',
                                attributes: {
                                    event: utils_1.getEventType(action)
                                }
                            }); })
                            : undefined
                    };
                });
            })
                .reduce(function (a, b) { return a.concat(b); })
            : [])).filter(Boolean)
    };
    return scxmlElement;
}
function fromMachine(machine) {
    var scxmlDocument = {
        declaration: { attributes: { version: '1.0', encoding: 'utf-8' } },
        elements: [
            { type: 'instruction', name: 'access-control', instruction: 'allow="*"' },
            {
                type: 'element',
                name: 'scxml',
                attributes: {
                    version: '1.0',
                    initial: machine.id
                }
            },
            stateNodeToSCXML(machine)
        ]
    };
    return xml_js_1.js2xml(scxmlDocument, { spaces: 2 });
}
exports.fromMachine = fromMachine;
console.log(fromMachine(lightMachine));
